unit io_test_unit1;
{Demo application for GPIO on Raspberry Pi}
{Inspired by the Python input/output demo application by Gareth Halfacree}
{written for the Raspberry Pi User Guide, ISBN 978-1-118-46446-5}
{ From https://wiki.freepascal.org/Lazarus_on_Raspberry_Pi#Accessing_external_hardware }
{
Modified for Beagle P9-15 (GPIO48) MAY2021 by:
John Dammeyer
Automation Artisans Inc.
johnd@autoartisans.com

  Expanded information reported on LogMemo so it's clear who is reporting what
  Note.  Run from command line with sudo in order to be able to work with /sys/class/gpio
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, FileUtil, Controls, Graphics, Dialogs, StdCtrls,
  Unix, BaseUnix;

type

  { TForm1 }

  TForm1 = class(TForm)
    LogMemo: TMemo;
    GPIO48ToggleBox: TToggleBox;
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure GPIO48ToggleBoxChange(Sender: TObject);
  private

  public

  end;

const
  PIN_48: PChar = '48';
  PIN_ON: PChar = '1';
  PIN_OFF: PChar = '0';
  OUT_DIRECTION: PChar = 'out';

var
  Form1: TForm1;
  gReturnCode: longint; {stores the result of the IO operation}

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormActivate(Sender: TObject);
  var
    fileDesc: integer;
  begin
    { Prepare P9-15, GPIO48 for access: }
    try
      fileDesc := fpopen('/sys/class/gpio/export', O_WrOnly);
      gReturnCode := fpwrite(fileDesc, PIN_48[0], 2);
      LogMemo.Lines.Add('Activate Open Result = '+IntToStr(gReturnCode));
    finally
      gReturnCode := fpclose(fileDesc);
      LogMemo.Lines.Add('Activate Close Result = '+IntToStr(gReturnCode));
    end;
    { Set P9-15, GPIO48 as output: }
    try
      fileDesc := fpopen('/sys/class/gpio/gpio48/direction', O_WrOnly);
      gReturnCode := fpwrite(fileDesc, OUT_DIRECTION[0], 3);
      LogMemo.Lines.Add('Set Direction Open Result = '+IntToStr(gReturnCode));
    finally
      gReturnCode := fpclose(fileDesc);
      LogMemo.Lines.Add('Set Direction Close Result = '+IntToStr(gReturnCode));
    end;
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
  var
    fileDesc: integer;
  begin
    { Free P9-15, GPIO48 }
    try
      fileDesc := fpopen('/sys/class/gpio/unexport', O_WrOnly);
      gReturnCode := fpwrite(fileDesc, PIN_48[0], 2);
      LogMemo.Lines.Add(IntToStr(gReturnCode));
    finally
      gReturnCode := fpclose(fileDesc);
      LogMemo.Lines.Add(IntToStr(gReturnCode));
    end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin

end;

procedure TForm1.GPIO48ToggleBoxChange(Sender: TObject);
  var
    fileDesc: integer;
  begin
    if GPIO48ToggleBox.Checked then
    begin
      { Switch P9-15, GPIO48 on: }
      try
        fileDesc := fpopen('/sys/class/gpio/gpio48/value', O_WrOnly);
        gReturnCode := fpwrite(fileDesc, PIN_ON[0], 1);
        LogMemo.Lines.Add('Pin ON Write Cmd = '+IntToStr(gReturnCode));
      finally
        gReturnCode := fpclose(fileDesc);
        LogMemo.Lines.Add('Pin ON Close = '+IntToStr(gReturnCode));
      end;
    end
    else
    begin
      { Switch P9-15, GPIO48 off: }
      try
        fileDesc := fpopen('/sys/class/gpio/gpio48/value', O_WrOnly);
        gReturnCode := fpwrite(fileDesc, PIN_OFF[0], 1);
        LogMemo.Lines.Add('Pin OFF Write Cmd = '+IntToStr(gReturnCode));
      finally
        gReturnCode := fpclose(fileDesc);
        LogMemo.Lines.Add('Pin OFF Close = '+IntToStr(gReturnCode));
      end;
    end;
end;

end.

